var isInt = require('is-integer');
var Node, Edge;

/**
 * Graph object, contains Nodes
 * @constructor
 */
var Graph = module.exports = function(){
	Node = require('./node.js');
	Edge = require('./edge.js');
	if(!(this instanceof Graph)){
		return new Graph();
	}
	this.nodes = [];
};

Graph.prototype = {
	type : "Graph",

	/**
	 * Add a node to the graph
	 * @memberof Graph#
	 * @param {Node} [node] - A Node object to add to the graph
	 * @returns {Graph} this
	 */
	addNode : function(node){
		if(typeof node === "undefined"){
			node = new Node();
		}
		if(!(node instanceof Node)){
			throw new Error("first argument to addNode() must be a Node object or undefined");
		}

		for(i=0;i<this.nodes.length;i++){
			if(this.nodes[i] === node){
				throw new Error("node already belongs to graph");
			}
		}

		this.nodes.push(node);
		node.graph = this;
		return this;
	},

	/**
	 * Get a node from the Graph
	 * @memberof Graph#
	 * @param {Int} id - id of the required node
	 * @returns {Node}
	 */
	getNode : function(id){
		if(!isInt(id)){
			throw new Error("first argument to getNode() must be an integer, " + id + " is not an integer");
		}
		return this.nodes[id] ? this.nodes[id] : null;
	},

	/**
	 * Get the number of nodes that belong to the graph
	 * @memberof Graph#
	 * @returns {Int}
	 */
	nNodes : function(){
		return this.nodes.length;
	},

	/**
	 * Add an edge between two nodes.
	 * @memberof Graph#
	 * @param {Node} node1 - start node
	 * @param {Node} node2 - end node
	 * @param {Int} [weight1=1] - edge weight going from node1 to node2
	 * @param {Int} [weight2=1] - edge weight going from node2 to node1
	 * @param {Bool} [isDirected=false] - is a directed edge
	 * @returns {Edge} Newly created Edge object
	 */
	addEdge : function(node1, node2, weight1, weight2){
		if(!(node1 instanceof Node && node2 instanceof Node)){
			throw new Error("first two arguments to addEdge() must be Node objects");
		}
		if(node1 === node2){
			throw new Error("nodes cannot be the same");
		}
		var inGraph = 0;
		for(var i=0;i<this.nodes.length;i++){
			if(this.nodes[i] === node1){
				inGraph++;
			}else if(this.nodes[i] === node2){
				inGraph++;
			}
		}
		if(inGraph < 2){
			throw new Error("first two arguments to addEdge() must be Node objects");
		}
		return new Edge(node1, node2, weight1, weight2);
	},

	/**
	 * @callback Graph~nodeCallback
	 * @param {Node} node
	 * @returns {*}
	 */

	/**
	 * Perform a callback on each node in the graph
	 * @memberof Graph#
	 * @param {Graph~nodeCallback} callback - Callback which is called with each node as it's context
	 * @returns {Array} array of callback results
	 */
	eachNode : function(callback){
		if(typeof callback !== "function"){
			throw new Error("first argument to eachNode() must be a function");
		}
		var return_array = [];
		for(var i=0; i<this.nodes.length; i++){
			return_array.push(callback.call(this.nodes[i]));
		}
		return return_array;
	},

	/**
	 * Create a number of nodes and add them all to the graph
	 * @memberof Graph#
	 * @param {Int} - Number of nodes to create
	 * @returns {Graph} this
	 */
	createNodes : function(number){
		if(!isInt(number)){
			throw new Error("first argument to createNodes() must be an integer");
		}
		for(var i=0; i<number; i++){
			this.addNode();
		}
		return this;
	},
};
