var Edge, Graph;
/**
 * Node object
 * @constructor
 */
var Node = module.exports = function(){
	Edge = require('./edge.js');
	Graph = require('./graph.js');
	if(!(this instanceof Node)){
		return new Node();
	}
	this.edges = [];
	this.graph = null;
};

Node.prototype = {
	type : "Node",

	/**
	 * @memberof Node#
	 * @returns {Graph} - Graph which this node belongs to
	 */
	getGraph : function(){
		return (this.graph instanceof Graph) ? this.graph : null;
	},

	/**
	 * Add an edge between this Node and another
	 * @memberof Node#
	 * @param {Node} node - end node
	 * @param {Int} [weight1=1] - outbound edge weight
	 * @param {Int} [weight2=1] - inbound edge weight
	 * @returns {Edge} Newly created Edge object
	 */
	addEdge : function(node, weight1, weight2){
		if(this === node){
			throw new Error("cannot connect a Node to itself");
		}
		return new Edge(this, node, weight1, weight2);
	},

	/**
	 * Add multiple edges from this node to others.
	 * @memberof Node#
	 * @param {Array} nodes - An array of nodes to connect this node to
	 * @param {Int} [weight1=1] - outbound edge weight
	 * @param {Int} [weight2=1] - inbound edge weight
	 * @returns {Node} this
	 */
	addEdges : function(nodes, weight1, weight2){
		if(!(nodes instanceof Array)){
			throw new Error("first argument to addEdges() must be an array of Node objects");
		}
		var i;
		for(i=0; i<nodes.length; i++){
			if(!(nodes[i] instanceof Node)){
				throw new Error("first argument to addEdges() must be an array of Node objects");
			}
		}
		for(i=0; i<nodes.length; i++){
			this.addEdge(nodes[i], weight1, weight2);
		}
		return this;
	},

	/**
	 * Get an edge which connects the current node to another
	 * @memberof Node#
	 * @param {Node} node - other node which the edge is connected to
	 * @returns {Edge} edge that connects the nodes
	 * @throws Will throw an error if the nodes are not connected
	 * @throws Will throw an error if this node object and node argument are the same
	 */
	getEdge : function(node){
		if(node === this){
			throw new Error("node passed to getEdge is the same node as this context");
		}
		if(!(node instanceof Node)){
			throw new Error("first argument to getEdge must be a node object");
		}
		for(var i=0;i<this.edges.length;i++){
			if(
				(this.edges[i].nodes[1] === node && !!this.edges[i].weights[0]) ||
				(this.edges[i].nodes[0] === node && !!this.edges[i].weights[1])
			){
				return this.edges[i];
			}
		}
		throw new Error("Nodes are not connected");
	},

	/**
	 * Get a node which is connected by an edge
	 * @memberof Node#
	 * @param {Edge} edge - Edge which connects this node to another
	 * @returns {Node} node which is connected to this node
	 * @throws Will throw an error if edge isn't connected to this node
	 */
	getNeighbour : function(edge){
		if(!(edge instanceof Edge)){
			throw new Error("first argument to getNeighbour() must be an Edge object");
		}
		for(var i=0;i<this.edges.length;i++){
			if(this.edges[i] === edge){
				if(this.edges[i].nodes[0] === this){
					return this.edges[i].nodes[1];
				}else if(this.edges[i].nodes[1] === this){
					return this.edges[i].nodes[0];
				}else{
					throw new Error("an edge was detected, but it's start and end didn't match this node");
				}
			}
		}
		return false;
	},

	/**
	 * Get an array of neighbours which are connected to this node
	 * @memberof Node#
	 * @returns {Array} Array of Node objects which are connected to this node
	 */
	getNeighbours : function(){
		var neighbours = [];
		for(var i=0;i<this.edges.length;i++){
			if(this.edges[i].nodes[0] === this){
				neighbours.push(this.edges[i].nodes[1]);
			}else if(this.edges[i].nodes[1] === this){
				neighbours.push(this.edges[i].nodes[0]);
			}else{
				throw new Error("an edge was detected, but it's start and end didn't match this node");
			}
		}
		return neighbours;
	},

	/**
	 * Get number of edges that are connected to this node
	 * @memberof Node#
	 * @returns {Int} number of edges
	 */
	nEdges : function(){
		return this.edges.length;
	},

	/**
	 * Is this node connected to another given node
	 * @memberof Node#
	 * @param {Node} node
	 * @returns {Bool}
	 * @throws Will throw an error if this node object and node argument are the same
	 */
	isNeighbour : function(node){
		if(!(node instanceof Node)){
			throw Error("first argument to isNeighbour() must be a Node object");
		}
		if(node === this){
			throw new Error("node passed to isNeighbour is the same node as this context");
		}

		try{
			var edge = this.getEdge(node);
			return edge instanceof Edge;
		}catch(e){
			return false;
		}
	},
};
