var assert = require('assert');
var bacon = require('../src/index.js');

describe('Graph', function(){
	describe('creation', function(){
		var graph = new bacon.Graph();
		it('should return an object',function(){
			assert.strictEqual("object", typeof graph);
		});
		it('object should have type = Graph', function(){
			assert.strictEqual("Graph", graph.type);
		});
		it('Graph() is same as new Graph()',function(){
			assert(bacon.Graph() instanceof bacon.Graph);
			assert(new bacon.Graph() instanceof bacon.Graph);
		});
		it('newly instantiated graph has no nodes', function(){
			assert.strictEqual(graph.nodes.length, 0);
		});
	});
	describe('addNode()', function(){
		it('expects a Node object or undefined as first argument', function(){
			var graph = new bacon.Graph();
			var variables = [
				null,
				123,
				true,
				false,
				"abcdef",
			];
			for(var i=0; i<variables.length; i++){
				assert.throws(function(){
					graph.addNode(variables[i]);
				}, /first argument to addNode\(\) must be a Node object or undefined/);
			}
			assert.doesNotThrow(function(){
				var graph = new bacon.Graph().addNode(new bacon.Node());
			});
			assert.doesNotThrow(function(){
				var graph = new bacon.Graph().addNode();
			});
		});
		it('should add one node to the graph', function(){
			var graph = new bacon.Graph();
			graph.addNode(new bacon.Node());
			assert.strictEqual(graph.nodes.length, 1);
		});
		it('returns the graph instance',function(){
			var graph = new bacon.Graph();
			assert.deepEqual(graph, graph.addNode(new bacon.Node()));
			assert.doesNotThrow(function(){
				graph.addNode(new bacon.Node())
				     .addNode(new bacon.Node())
					 .addNode(new bacon.Node());
			});
		});
		it('error when trying to add the same node twice',function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			graph.addNode(node);
			assert.throws(function(){
				graph.addNode(node);
			}, /node already belongs to graph/);
		});
	});
	describe('createNodes()', function(){
		it('expects an integer as it\'s only argument', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();

			var variables = [
				null,
				undefined,
				true,
				false,
				"abcdef",
				1.1,
				node,
			];
			for(var i=0; i<variables.length; i++){
				assert.throws(function(){
					graph.createNodes(variables[i]);
				}, /first argument to createNodes\(\) must be an integer/);
			}
			assert.doesNotThrow(function(){
				graph.createNodes(10);
			});
		});
		it('creates N amount of nodes and adds them to the graph', function(){
			var graph = new bacon.Graph();
			graph.createNodes(5);
			assert.equal(graph.nNodes(), 5);
			graph.createNodes(10);
			assert.equal(graph.nNodes(), 15);
		});
		it('returns the graph object', function(){
			var graph = new bacon.Graph();
			assert.deepEqual(graph, graph.createNodes(5));
		});
	});
	describe('getNode()', function(){
		it('expects an integer id as it\'s only argument', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var variables = [
				null,
				undefined,
				true,
				false,
				"abcdef",
				1.1,
			];
			for(var i=0; i<variables.length; i++){
				assert.throws(function(){
					graph.getNode(variables[i]);
				}, /first argument to getNode\(\) must be an integer/);
			}
			assert.doesNotThrow(function(){
				graph.getNode(1);
			});
		});
		it('returns null if no node is found',function(){
			var graph = new bacon.Graph();
			assert.strictEqual(null, graph.getNode(100));
		});
		it('should fetch a node by id', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			graph.addNode(node);
			assert.deepEqual(graph.getNode(0), node);
		});
		it('gets the right node by id', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node).addNode(node2);
			assert.deepEqual(graph.getNode(1), node2);
			assert.deepEqual(graph.getNode(0), node);
		});
	});
	describe('nNodes()', function(){
		it('returns the number of nodes', function(){
			var graph = new bacon.Graph();
			assert.strictEqual(0, graph.nNodes());
			graph.addNode(new bacon.Node());
			assert.strictEqual(1, graph.nNodes());
			graph.addNode(new bacon.Node());
			graph.addNode(new bacon.Node());
			assert.strictEqual(3, graph.nNodes());
		});
	});
	describe('addEdge()', function(){
		it('adds an edge between two of a graph\'s nodes', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node).addNode(node2);
			graph.addEdge(node, node2);
			assert.equal(true, node.getEdge(node2) instanceof bacon.Edge);
		});
		it('expects two node objects as arguments', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node).addNode(node2);
			assert.doesNotThrow(function(){
				graph.addEdge(node, node2);
			});
			assert.throws(function(){
				graph.addEdge(node, null);
			}, /first two arguments to addEdge\(\) must be Node objects/);
			assert.throws(function(){
				graph.addEdge(node, {});
			}, /first two arguments to addEdge\(\) must be Node objects/);
			assert.throws(function(){
				graph.addEdge(null, node);
			}, /first two arguments to addEdge\(\) must be Node objects/);
			assert.throws(function(){
				graph.addEdge({}, node);
			}, /first two arguments to addEdge\(\) must be Node objects/);
			assert.throws(function(){
				graph.addEdge(node);
			}, /first two arguments to addEdge\(\) must be Node objects/);
			assert.throws(function(){
				graph.addEdge({},{});
			}, /first two arguments to addEdge\(\) must be Node objects/);
		});
		it('both nodes must belong to this graph', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			var node3 = new bacon.Node();
			graph.addNode(node).addNode(node2);
			assert.doesNotThrow(function(){
				graph.addEdge(node, node2);
			});
			assert.throws(function(){
				graph.addEdge(node, node3);
			}, /first two arguments to addEdge\(\) must be Node objects/);
			assert.throws(function(){
				graph.addEdge(node3, node2);
			}, /first two arguments to addEdge\(\) must be Node objects/);
		});
		it('order of node1 and node2 shouldn\'t affect the returned edge', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node1).addNode(node2);
			graph.addEdge(node1, node2);

			var edge1 = node1.getEdge(node2);
			var edge2 = node2.getEdge(node1);

			assert.deepEqual(node1, node2);
		});
		it('empty or false third argument defines edge weight = 1 in both directions', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			var node3 = new bacon.Node();
			graph.addNode(node1).addNode(node2).addNode(node3);
			graph.addEdge(node1, node2);
			graph.addEdge(node2, node3, false);

			var edge12 = node1.getEdge(node2);
			var edge23 = node2.getEdge(node3);
			assert.deepEqual(edge12.weights, [1, 1]);
			assert.deepEqual(edge23.weights, [1, 1]);

		});
		it('third argument = true, defines directed edge weight = 1 from node1 to node2', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node1).addNode(node2);
			graph.addEdge(node1, node2, true);

			var edge = node1.getEdge(node2);
			var halfEdge1 = edge.getHalfEdge(node2);
			var halfEdge2 = edge.getHalfEdge(node1);

			assert.equal(halfEdge1.weight, 1);
			assert.equal(halfEdge2, null);
		});
		it('third argument is numeric, defines weighted edge', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node1).addNode(node2);
			graph.addEdge(node1, node2, 9);

			var edge = node1.getEdge(node2);
			var halfEdge1 = edge.getHalfEdge(node2);
			var halfEdge2 = edge.getHalfEdge(node1);

			assert.equal(halfEdge1.weight, 9);
			assert.equal(halfEdge2.weight, 9);
		});
		it('third argument is numeric, fourth argument = false, defines weighted edge', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node1).addNode(node2);
			graph.addEdge(node1, node2, 42, false);

			var edge = node1.getEdge(node2);
			var halfEdge1 = edge.getHalfEdge(node2);
			var halfEdge2 = edge.getHalfEdge(node1);

			assert.equal(halfEdge1.weight, 42);
			assert.equal(halfEdge2.weight, 42);
		});
		it('third argument is numeric, fourth argument = true, defines directional weighted edge', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node1).addNode(node2);
			graph.addEdge(node1, node2, 42, true);

			var edge = node1.getEdge(node2);
			var halfEdge1 = edge.getHalfEdge(node2);
			var halfEdge2 = edge.getHalfEdge(node1);

			assert.equal(halfEdge1.weight, 42);
			assert.equal(halfEdge2, null);

		});
		it('third and fourth arguments are numeric, defines edge weights in both directions', function(){
			var graph = new bacon.Graph();
			var node1 = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node1).addNode(node2);
			graph.addEdge(node1, node2, 9, 66);

			var edge = node1.getEdge(node2);
			var halfEdge1 = edge.getHalfEdge(node2);
			var halfEdge2 = edge.getHalfEdge(node1);

			assert.equal(halfEdge1.weight, 9);
			assert.equal(halfEdge2.weight, 66);
		});
	});
	describe('eachNode()', function(){
		it('expects a function as first argument', function(){
			var graph = new bacon.Graph();
			assert.doesNotThrow(function(){
				graph.eachNode(function(){});
			});
			assert.throws(function(){
				graph.eachNode(null);
			}, /first argument to eachNode\(\) must be a function/);
			assert.throws(function(){
				graph.eachNode({});
			}, /first argument to eachNode\(\) must be a function/);
		});
		it('performs a function on each node in the graph', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			graph.addNode(node);
			graph.addNode(node2);

			var functionRan = false;
			var function2Ran = false;

			graph.eachNode(function(){
				if(functionRan){
					function2Ran = true;
				}
				functionRan = true;
			});
			assert(functionRan);
			assert(function2Ran);
		});
		it('callback is passed node as it\'s context', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			graph.addNode(node);
			var context;
			graph.eachNode(function(){
				context = this;
			});
			assert.deepEqual(node, context);
		});
		it('returns an array of function returns', function(){
			var graph = new bacon.Graph();
			var node = new bacon.Node();
			var node2 = new bacon.Node();

			node.x = "one";
			node2.x = "two";

			graph.addNode(node);
			graph.addNode(node2);

			var returns = graph.eachNode(function(){
				return this.x;
			});
			assert.deepEqual(["one","two"], returns);
		});
	});
});
