var assert = require('assert');
var bacon = require('../src/index.js');

describe('Node', function(){
	describe('creation', function(){
		var node = new bacon.Node();
		it('should return an object',function(){
			assert.strictEqual("object", typeof node);
		});
	});
	describe('getGraph()', function(){
		var graph = new bacon.Graph();
		var node = new bacon.Node();
		var node2 = new bacon.Node();
		graph.addNode(node2);
		it('returns null if node not connected to graph', function(){
			assert.strictEqual(null, node.getGraph());
		});
		it('returns Graph that the node belongs to', function(){
			assert.deepEqual(graph, node2.getGraph());
		});
	});
	describe('addEdge(node)', function(){
		it('expects a node object as the only argument', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			var variables = [
				'',
				null,
				123,
				undefined,
				{},
				//new bacon.Edge()
			];
			for(var i; i<variables.length; i++){
				assert.throws(function(){
					node.addEdge(variables[i]);
				}, /first argument to addEdge\(\) must be a Node object/);
			}
			assert.doesNotThrow(function(){
				node.addEdge(node2);
			});
		});
		it('throws an error when trying to connect a node to itself', function(){
			var node = new bacon.Node();
			assert.throws(function(){
				node.addEdge(node);
			}, /cannot connect a Node to itself/);
		});
		it('returns resulting edge object', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			assert(node.addEdge(node2) instanceof bacon.Edge);
		});
		it('allows new edge, even if nodes are already connected', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			assert.doesNotThrow(function(){
				node.addEdge(node2);
				node.addEdge(node2);
			});
		});
	});
	describe('addEdges()', function(){
		it('expects an array of nodes', function(){
			var node = new bacon.Node();
			assert.throws(function(){
				node.addEdges([1,2,3,4]);
			}, /first argument to addEdges\(\) must be an array of Node object/);
			assert.throws(function(){
				node.addEdges({one:new bacon.Node(), two:new bacon.Node()});
			}, /first argument to addEdges\(\) must be an array of Node object/);
			assert.throws(function(){
				node.addEdges(null);
			}, /first argument to addEdges\(\) must be an array of Node object/);
			assert.throws(function(){
				node.addEdges();
			}, /first argument to addEdges\(\) must be an array of Node object/);
			assert.throws(function(){
				node.addEdges([
					new bacon.Node(),
					null,
					new bacon.Node(),
				]);
			}, /first argument to addEdges\(\) must be an array of Node object/);
			assert.doesNotThrow(function(){
				var nodes = [
					new bacon.Node(),
					new bacon.Node(),
					new bacon.Node(),
					new bacon.Node(),
				];
				node.addEdges(nodes);
				node.addEdges([]);
			});
		});
		it('returns the original node', function(){
			var node = new bacon.Node();
			var nodes = [
				new bacon.Node(),
				new bacon.Node(),
				new bacon.Node(),
				new bacon.Node(),
			];
			assert.deepEqual(node, node.addEdges(nodes));
		});
	});
	describe('getEdge()', function(){
		it('expects node as it\'s only argument', function(){
			var node = new bacon.Node();
			var variables = [
				'',
				null,
				123,
				undefined,
				{},
			];
			for(var i; i<variables.length; i++){
				assert.throws(function(){
					node.getEdge(variables[i]);
				}, /first argument to getEdge\(\) must be a Node object/);
			}
		});
		it('returns an edge between two nodes', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			var edge = node.addEdge(node2);
			assert.deepEqual(edge, node.getEdge(node2));
		});
		it('throws error if nodes aren\'t connected', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			assert.throws(function(){
				node.getEdge(node2);
			}, /Nodes are not connected/);
		});
	});
	describe('nEdges()', function(){
		it('returns number of edges belonging to a node', function(){
			var node = new bacon.Node();
			var neighbours = [];
			for(var i=0; i<4; i++){
				neighbours[i] = new bacon.Node();
			}

			assert.strictEqual(0, node.nEdges());
			node.addEdge(neighbours[0]);
			assert.strictEqual(1, node.nEdges());
			node.addEdge(neighbours[1])
			node.addEdge(neighbours[2]);
			assert.strictEqual(3, node.nEdges());
			node.addEdge(neighbours[3]);
			assert.strictEqual(4, node.nEdges());
		});
	});
	describe('getNeighbour', function(){
		it('gets a node\'s neighbour given a edge', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			var edge = node.addEdge(node2);
			assert.deepEqual(node2, node.getNeighbour(edge));
		});
		it('expects edge as it\'s only argument', function(){
			var node = new bacon.Node();
			var variables = [
				'',
				null,
				123,
				undefined,
				{},
			];
			for(var i=0; i<variables.length; i++){
				assert.throws(function(){
					node.getNeighbour(variables[i]);
				}, /first argument to getNeighbour\(\) must be an Edge object/);
			}
			assert.doesNotThrow(function(){
				node.getNeighbour(new bacon.Edge(node, new bacon.Node()));
			});
		});
		it('returns false if the edge and node are not connected', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			var node3 = new bacon.Node();
			var edge = node2.addEdge(node3);
			assert.strictEqual(false, node.getNeighbour(edge));
		});
	});
	describe('getNeighbours', function(){
		it('returns an array of neighbours', function(){
			var node = new bacon.Node();
			var node2 = new bacon.Node();
			node.addEdge(node2);
			assert.deepEqual([node2],node.getNeighbours());
			assert.deepEqual([node],node2.getNeighbours());
		});
		it('returns an empty array if no neighbours exist', function(){
			var node = new bacon.Node();
			assert.deepEqual([], node.getNeighbours());
		});
	});
	describe('isNeighbour', function(){
		var node = new bacon.Node();
		var node2 = new bacon.Node();
		var node3 = new bacon.Node();
		node.addEdge(node2);
		node3.addEdge(node2);
		it('returns true for connected nodes', function(){
			assert.strictEqual(true, node.isNeighbour(node2));
			assert.strictEqual(true, node2.isNeighbour(node));
			assert.strictEqual(true, node3.isNeighbour(node2));
			assert.strictEqual(true, node2.isNeighbour(node3));
		});
		it('returns false for unconnected nodes', function(){
			assert.strictEqual(false, node.isNeighbour(node3));
			assert.strictEqual(false, node3.isNeighbour(node));
		});
		it('expects a node as it\'s first argument', function(){
			assert.throws(function(){
				node.isNeighbour();
			}, /first argument to isNeighbour\(\) must be a Node object/);
			assert.throws(function(){
				node.isNeighbour(null);
			}, /first argument to isNeighbour\(\) must be a Node object/);
			assert.throws(function(){
				node.isNeighbour({});
			}, /first argument to isNeighbour\(\) must be a Node object/);
		});
	});
});
